@@ -1,6 +1,7 @@
 #! /usr/bin/python3
-import aiohttp, bs4, asyncio, discord
-from io import BytesIO
+import aiohttp, asyncio, discord
+from lxml import etree
+from io import BytesIO, StringIO
 from PIL import Image
 import os, pkgutil
 sfilter=pkgutil.get_data(__package__, 'safesearchfilter.txt').decode('utf-8').replace("\r","").split("\n")
@@ -12,7 +13,26 @@ class ImageDataNotFound(Exception):
 class BadRequest(Exception):
     '''Non 200 response code'''
     pass
+class VideoList:
+    '''Gives a list of video urls, 
+    100x more performant than manually 
+    querying results using Video Object
+    
+    returns: list of video urls'''
+
+    def __init__(self):
+        self.videos=[]
+    async def search(self, query):
+        async with aiohttp.ClientSession() as session:
+            async with session.get(f"https://youtube.com/search?q={query}") as r:
+                if r.status==200:
+                    html=await r.text()
+                    data=etree.HTML(html)
+                    self.videos=["https://youtube.com"+url for url in data.xpath("//div[contains(@class,'yt-lockup-thumbnail')]/a/@href")] # Parses data to extracts urls of all videos
+                else:      
+                    raise BadRequest  
 class Video:
+    ''' Search for a youtube video'''
     def __init__(self,advanced=False):
         if SAFESEARCH:
             self.__censoredcontent=sfilter
@@ -43,38 +63,36 @@ class Video:
             async with session.get(queryurl) as r:
                 if r.status == 200:
                     text=await r.text()
+                    
                 else:
                     raise BadRequest
-        # Parse data using bs4
-        soup = bs4.BeautifulSoup(text, 'html.parser')
-        contents=[ d for d in soup.find_all('div') if d.has_attr('class') and 'yt-lockup-dismissable' in d['class']]
-        if not contents:
+        data=etree.HTML(text)
+        try:
+            self.url="https://youtube.com"+data.xpath("//div[contains(@class,'yt-lockup-thumbnail')]/a/@href")[index]
+            self.thumbnail=data.xpath("//div[contains(@class,'yt-lockup-thumbnail')]/a//img/@src")[index] # May not work after the 5th result because of how youtube loads it client sided
+            self.title=data.xpath("//h3[contains(@class,'yt-lockup-title')]/a")[index].text
+        except IndexError:
             print(f"Couldn't fetch video, retrying...")
             await self.search(name,index,retries=retries-1)
-        try:
-            self.url='http://www.youtube.com'+contents[index].a["href"]
-            self.thumbnail=contents[index].img["src"]
-            self.title=contents[index].h3.a["title"]
-            if self.advanced:
-                async with aiohttp.ClientSession() as session:
-                    async with session.get(self.url) as r:
-                        if r.status == 200:
-                            text=await r.text()
-                        else:
-                            raise BadRequest     
-                soup = bs4.BeautifulSoup(text, 'html.parser')
-                self.views=soup.select_one(".watch-view-count").text.strip()
-                self.published_on=soup.select_one(".watch-time-text").text.strip()
-                __creatorinfo=soup.select_one(".yt-user-info")
-                self.channel="https://youtube.com"+__creatorinfo.a["href"]
-                self.creator=__creatorinfo.a.text.strip()
-                __likebtn=soup.select_one('.like-button-renderer')
-                self.likes=__likebtn.select_one(".like-button-renderer-like-button").text.strip()
-                self.dislikes=__likebtn.select_one(".like-button-renderer-dislike-button").text.strip()
-                self.description=soup.select_one("#watch-description-text").text
-        except Exception as ex:
-            print(f"Error: {ex}, retrying...")
-            await self.search(name,index,retries=retries-1)
+        if self.advanced:
+            async with aiohttp.ClientSession() as session:
+                async with session.get(self.url) as r:
+                    if r.status == 200:
+                        adv=await r.text()
+                    else:
+                        raise BadRequest    
+            advdata=etree.HTML(adv)
+            try:
+                self.views=advdata.xpath("//div[contains(@class,'watch-view-count')]")[0].text
+                self.published_on=advdata.xpath("//strong[contains(@class,'watch-time-text')]")[0].text 
+                self.channel="https://youtube.com"+advdata.xpath("//div[contains(@class,'yt-user-info')]/a/@href")[0]
+                self.creator=advdata.xpath("//div[contains(@class,'yt-user-info')]/a")[0].text
+                self.likes=advdata.xpath("//button[contains(@class,'like-button-renderer-like-button')]/span")[0].text#like-button-renderer-like-button
+                self.dislikes=advdata.xpath("//button[contains(@class,'like-button-renderer-dislike-button')]/span")[0].text#like-button-renderer-dislike-button
+                self.description=advdata.xpath("//p[contains(@id,'eow-description')]")[0].text#eow-description
+            except IndexError:
+                print(f"Couldn't fetch advanced data, retrying...")
+                await self.search(name,index,retries=retries-1)
     async def image(self):
         if not self.thumbnail:
             raise ImageDataNotFound
